package digital.laboratory.platform.inspection.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import com.deepoove.poi.xwpf.NiceXWPFDocument;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.inspection.dto.TestRecordSampleDataDocDTO;
import digital.laboratory.platform.inspection.entity.TestRecordInstrument;
import digital.laboratory.platform.inspection.entity.TestRecordReagent;
import digital.laboratory.platform.inspection.entity.TestRecordSampleData;
import digital.laboratory.platform.inspection.entity.TestRecordSampleDataExpand;
import digital.laboratory.platform.inspection.enums.BusinessType;
import digital.laboratory.platform.inspection.enums.TestRecordFileUrl;
import digital.laboratory.platform.inspection.service.*;
import digital.laboratory.platform.inspection.vo.IonPairAndCEVO;
import digital.laboratory.platform.inspection.vo.TestRecordReagentVO;
import digital.laboratory.platform.inspetion.api.entity.EntrustInfo;
import digital.laboratory.platform.inspetion.api.entity.SampleInfo;
import digital.laboratory.platform.inspetion.api.entity.TargetObject;
import digital.laboratory.platform.inspetion.api.entity.TestRecord;
import digital.laboratory.platform.sys.entity.Drug;
import digital.laboratory.platform.sys.enums.entrust.EntrustBiologyType;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.BeanUtils;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author ChenJiangBao
 * @version 1.0
 * @description: 生成检验记录相关接口实现类
 * @date 2025/3/18 14:50
 */
@Service
public class InspectRecordServiceImpl implements InspectRecordService {

    @Resource
    private OssFile ossFile;

    @Resource
    private EntrustInfoService entrustInfoService;

    @Resource
    private TestRecordService testRecordService;

    @Resource
    private TestRecordInstrumentService testRecordInstrumentService;

    @Resource
    private TestRecordSampleDataService testRecordSampleDataService;

    @Resource
    private TestRecordReagentService testRecordReagentService;

    @Resource
    private SampleInfoService sampleInfoService;

    @Resource
    private TestRecordSampledataExpandService testRecordSampledataExpandService;

    @Resource
    private RemoteWord2PDFService remoteWord2PDFService;

    /**
     * 生成检验记录-贵阳禁毒
     *
     * @param businessId   委托id
     * @param materialType 标注当前委托的检材是尿液还是毛发
     * @return
     * @throws Exception
     */
    @Override
    public R buildInspectionRecord(String businessId, String materialType) throws Exception {
        EntrustInfo entrustInfo = entrustInfoService.getById(businessId);
        if (entrustInfo == null) {
            return R.ok(false, "委托信息不存在！");
        }
        if (entrustInfo.getBusinessType().equals(BusinessType.BOINT_CASE.getBusinessType())) {
            return R.ok(this.buildInVivoDocFile(entrustInfo, materialType), "生成成功!");
        }
        return R.ok(this.generateCommonDrugInpectRecord(businessId));
    }

    @Override
    public void previewInspectionRecord(String businessId, String materialType, HttpServletResponse servletResponse) throws Exception {

        String path = (String) this.buildInspectionRecord(businessId, materialType).getData();
        java.io.ByteArrayOutputStream fosWord = new java.io.ByteArrayOutputStream();
        ossFile.fileGet(path, fosWord);
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();
        // 转换临时目录中的 word 文档为 PDF
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", "检验记录" + ".docx", "image/jpg", fisWord);
        String pdfFilePath = path.substring(0, path.lastIndexOf(".")) + ".pdf";
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();
        org.apache.commons.io.output.ByteArrayOutputStream outPDF = new org.apache.commons.io.output.ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();
        ossFile.fileSave(pdfFilePath, isPDF);
        isPDF.close();
        ossFile.fileGet(pdfFilePath, servletResponse.getOutputStream());
        System.out.println(String.format("转换为 PDF 结束"));
    }

    /**
     * 生成生物样本检验记录数据map
     *
     * @param entrustInfo  委托实体信息
     * @param materialType 检材类型
     * @return
     * @throws Exception
     */
    public Map<String, Object> buildInVivoRecordData(EntrustInfo entrustInfo, String materialType) throws Exception {


        // 获取检验记录信息
        TestRecord testRecord = testRecordService.getTestRecordByBusinessId(entrustInfo.getId());

        if (testRecord == null) {
            throw new Exception("未找到检验记录信息");
        }

        // 生成基础数据
        HashMap<String, Object> data = buildCommonInspectRecordDocMap(entrustInfo, testRecord, materialType);


        // 获取样品数据
        List<TestRecordSampleData> dataList = testRecordSampleDataService.list(
                Wrappers.<TestRecordSampleData>lambdaQuery().eq(TestRecordSampleData::getTestId, testRecord.getId()));

        if (CollectionUtils.isEmpty(dataList)) {
            data.put("dataDtos", Collections.emptyList());
            data.put("sampleSize", 0);
            data.put("vo", testRecord);
            return data;
        }

        // 批量查询所有样品扩展数据，避免多次查询数据库
        Map<String, List<TestRecordSampleDataExpand>> dataExpandMap = testRecordSampledataExpandService.list(
                Wrappers.<TestRecordSampleDataExpand>lambdaQuery()
                        .in(TestRecordSampleDataExpand::getTestDataId, dataList.stream().map(TestRecordSampleData::getId).collect(Collectors.toList()))
        ).stream().collect(Collectors.groupingBy(TestRecordSampleDataExpand::getTestDataId));

        // 根据化合物名称分组
        Map<String, List<TestRecordSampleData>> dataMap = dataList.stream()
                .collect(Collectors.groupingBy(TestRecordSampleData::getCompoundName));

        List<TestRecordSampleDataDocDTO> dataDtos = new ArrayList<>();

        // 遍历化合物组
        for (Map.Entry<String, List<TestRecordSampleData>> entry : dataMap.entrySet()) {
            String compoundName = entry.getKey();
            List<TestRecordSampleData> list = entry.getValue();

            // 添加空白数据
            TestRecordSampleDataDocDTO blankVo = new TestRecordSampleDataDocDTO();
            blankVo.setName("空白" + materialType);
            blankVo.setCompoundName(compoundName);
            blankVo.setPTargetRtTime("/");
            blankVo.setPRtTimeError("/");
            blankVo.setPRtTimeWithinError("否");
            blankVo.setPIonAbundanceRatio("/");
            blankVo.setPIonAbundanceRatioError("/");
            blankVo.setPIonAbundanceRatioWithinError("否");
            blankVo.setPIsDetected("否");
            dataDtos.add(blankVo);

            // 根据样品类型分组
            Map<String, List<TestRecordSampleData>> map = list.stream()
                    .collect(Collectors.groupingBy(TestRecordSampleData::getSampleType));

            // 处理标准样品（STD）
            if (map.containsKey("STD")) {
                TestRecordSampleData std = map.get("STD").get(0);
                TestRecordSampleDataDocDTO stdVo = new TestRecordSampleDataDocDTO();
                BeanUtils.copyProperties(std, stdVo);
                List<TestRecordSampleDataExpand> expandList = dataExpandMap.get(std.getId());
                if (expandList != null) {
                    for (TestRecordSampleDataExpand expand : expandList) {
                        if (!expand.getBasePeak()) {
                            stdVo.setPIonAbundanceRatio(expand.getIonAbundanceRatio().toString());
                            stdVo.setPIonAbundanceRatioError("/");
                            stdVo.setPIonAbundanceRatioWithinError("/");
                            break; // 只取第一个符合条件的扩展数据
                        }
                    }
                }
                stdVo.setName("空白" + materialType + "加标");
                stdVo.setPTargetRtTime(std.getTargetRtTime().toString());
                stdVo.setPRtTimeError("/");
                stdVo.setPRtTimeWithinError("/");
                stdVo.setPIsDetected("是");
                dataDtos.add(stdVo);
            }

            // 处理检材样品（Analyte）
            if (map.containsKey("Analyte")) {
                List<TestRecordSampleData> analyte = map.get("Analyte");
                analyte.sort(testRecordSampleDataService.getSortBySampleNo());

                List<TestRecordSampleDataDocDTO> dataVOS = new ArrayList<>();
                for (int i = 0; i < analyte.size(); i++) {
                    TestRecordSampleData item = analyte.get(i);
                    TestRecordSampleDataDocDTO vo = new TestRecordSampleDataDocDTO();
                    BeanUtils.copyProperties(item, vo);
                    vo.setPTargetRtTime(vo.getTargetRtTime().toString());
                    vo.setPRtTimeError(vo.getRtTimeError().toString());
                    vo.setPRtTimeWithinError(vo.getRtTimeWithinError().toString());
                    vo.setPIsDetected(item.getIsDetected() == 1 ? "是" : "否");

                    List<TestRecordSampleDataExpand> expandList = dataExpandMap.get(item.getId());
                    if (expandList != null) {
                        for (TestRecordSampleDataExpand expand : expandList) {
                            if (!expand.getBasePeak()) {
                                vo.setPIonAbundanceRatio(expand.getIonAbundanceRatio().toString());
                                vo.setPIonAbundanceRatioError(expand.getIonAbundanceRatioError().toString());
                                vo.setPIonAbundanceRatioWithinError(expand.getIonAbundanceRatioWithinError());
                                break; // 只取第一个符合条件的扩展数据
                            }
                        }
                    }
                    // 重新命名样品
                    vo.setName((dataList
                            .stream()
                            .collect(Collectors.groupingBy(TestRecordSampleData::getSampleNo))
                            .keySet()
                            .size() == 1) ? "检材样品" : (i + 1) + "号检材样品");
                    dataVOS.add(vo);
                }
                dataDtos.addAll(dataVOS);
            }
        }
        // 加入图谱序号值
        int indexNum = 1;
        for (int i = 0; i < dataDtos.size(); i++) {
            dataDtos.get(i).setIndexNum(String.valueOf(indexNum++));
        }
        this.buildIonPairAndCE(data, testRecordReagentService
                .list(Wrappers.<TestRecordReagent>lambdaQuery()
                        .in(TestRecordReagent::getId, testRecord.getReagentConsumablesList())
                        .eq(TestRecordReagent::getCategory, "标准物质"))
                .stream()
                .map(item -> item.getId())
                .collect(Collectors.toList()));
        // 返回处理后的数据
        data.put("dataDtos", dataDtos);
        data.put("sampleSize", dataDtos.size() / 2);
        data.put("inspectOpinion", this.buildInspectOpinion(dataList));
        return data;
    }

    public Map<String, Object> buildInVitroRecordData(EntrustInfo entrustInfo, TestRecord testRecord) {
        Map<String, Object> data = new HashMap<>();

        // 1️⃣  按化合物名称分组 TestRecordSampleData
        Map<String, List<TestRecordSampleData>> compoundSampleDataMap = testRecordSampleDataService
                .lambdaQuery()
                .eq(TestRecordSampleData::getTestId, testRecord.getId())
                .list()
                .stream()
                .collect(Collectors.groupingBy(TestRecordSampleData::getCompoundName));

        // 2️⃣  过滤出 "标准物质" 类别的试剂，并转换为 Map (key=药品名称, value=试剂VO)
        List<String> reagentConsumablesList = testRecord.getReagentConsumablesList();
        Map<String, TestRecordReagentVO> reagentMap = reagentConsumablesList.stream()
                .map(testRecordReagentService::getVOById)
                .filter(vo -> "标准物质".equals(vo.getCategory())) // 仅保留 "标准物质"
                .collect(Collectors.toMap(vo -> vo.getDrug().getName(), vo -> vo));

        List<TestRecordSampleDataDocDTO> dataDtos = new ArrayList<>();

        // 3️⃣ 遍历每种化合物，处理实验数据
        for (Map.Entry<String, List<TestRecordSampleData>> entry : compoundSampleDataMap.entrySet()) {
            String compoundName = entry.getKey();
            List<TestRecordSampleData> sampleDataList = entry.getValue();

            // 按 sampleType 分组（如 STD, Analyte）
            Map<String, List<TestRecordSampleData>> sampleTypeMap = sampleDataList.stream()
                    .collect(Collectors.groupingBy(TestRecordSampleData::getSampleType));

            // 获取标准样本（STD）
            List<TestRecordSampleData> stdList = sampleTypeMap.get("STD");
            if (stdList == null || stdList.isEmpty()) {
                continue; // 没有 STD 样本，跳过该化合物
            }
            TestRecordSampleData stdSample = stdList.get(0);

            // 4️⃣  获取 STD 样本的扩展数据 (以 MassToChargeRatio 为 key)
            Map<BigDecimal, TestRecordSampleDataExpand> expandMap = testRecordSampledataExpandService.lambdaQuery()
                    .eq(TestRecordSampleDataExpand::getTestDataId, stdSample.getId())
                    .list()
                    .stream()
                    .collect(Collectors.toMap(TestRecordSampleDataExpand::getMassToChargeRatio, Function.identity()));

            // 5️⃣  获取当前化合物对应的试剂，提取特征离子
            TestRecordReagentVO reagentVO = reagentMap.get(compoundName);
            if (reagentVO == null || reagentVO.getDrug().getCharacteristicIons() == null) {
                continue; // 没有对应试剂或特征离子数据，跳过
            }
            String[] peaks = reagentVO.getDrug().getCharacteristicIons().split("、");

            // 6️⃣  处理 "标样" 数据 (STD)
            for (int i = 0; i < peaks.length; i++) {
                String peak = (i == 0) ? peaks[i].substring(0, peaks[i].length() - 1) : peaks[i];
                TestRecordSampleDataExpand expand = expandMap.get(new BigDecimal(peak));

                if (expand != null) {
                    TestRecordSampleDataDocDTO dto = new TestRecordSampleDataDocDTO();
                    BeanUtils.copyProperties(stdSample, dto);
                    dto.setName("标样");
                    dto.setPTargetRtTime(dto.getTargetRtTime().toString());
                    dto.setPRtTimeError("±1");
                    dto.setPRtTimeWithinError("/");
                    dto.setPMassToChargeRatio(i == 0 ? peak + "（基峰）" : peak);
                    dto.setPIonAbundanceRatio(expand.getIonAbundanceRatio().toString());
                    dto.setPIonAbundanceRatioError(expand.getIonAbundanceRatioError().toString());
                    dto.setPIonAbundanceRatioWithinError(expand.getIonAbundanceRatioWithinError());
                    dto.setPIsDetected("/");
                    dataDtos.add(dto);
                }
            }

            // 7️⃣  处理 "标样空白" 数据
            for (String peak : peaks) {
                TestRecordSampleDataDocDTO dto = new TestRecordSampleDataDocDTO();
                dto.setName("标样空白");
                dto.setCompoundName(compoundName);
                dto.setPTargetRtTime("/");
                dto.setPRtTimeError("/");
                dto.setPRtTimeWithinError("/");
                dto.setPMassToChargeRatio(peak.equals(peaks[0]) ? peak + "（基峰）" : peak);
                dto.setPIonAbundanceRatio("/");
                dto.setPIonAbundanceRatioError("/");
                dto.setPIonAbundanceRatioWithinError(peak.equals(peaks[0]) ? "/" : "否");
                dto.setPIsDetected("/");
                dataDtos.add(dto);
            }

            // 8️⃣  处理 "Analyte" 样本
            List<TestRecordSampleData> analyteList = sampleTypeMap.get("Analyte");
            if (analyteList != null) {
                analyteList.sort(testRecordSampleDataService.getSortBySampleNo()); // 按样本编号排序

                for (TestRecordSampleData analyte : analyteList) {
                    Map<BigDecimal, TestRecordSampleDataExpand> analyteExpandMap = testRecordSampledataExpandService.lambdaQuery()
                            .eq(TestRecordSampleDataExpand::getTestDataId, analyte.getId())
                            .list()
                            .stream()
                            .collect(Collectors.toMap(TestRecordSampleDataExpand::getMassToChargeRatio, Function.identity()));

                    for (int i = 0; i < peaks.length; i++) {
                        String peak = (i == 0) ? peaks[i].substring(0, peaks[i].length() - 1) : peaks[i];
                        TestRecordSampleDataExpand expand = analyteExpandMap.get(new BigDecimal(peak));

                        if (expand != null) {
                            TestRecordSampleDataDocDTO dto = new TestRecordSampleDataDocDTO();
                            BeanUtils.copyProperties(analyte, dto);
                            dto.setPTargetRtTime(dto.getTargetRtTime().toString());
                            dto.setPRtTimeError(dto.getRtTimeWithinError());
                            dto.setPRtTimeWithinError(dto.getRtTimeWithinError());
                            dto.setPMassToChargeRatio(i == 0 ? peak + "（基峰）" : peak);
                            dto.setPIonAbundanceRatio(expand.getIonAbundanceRatio().toString());
                            dto.setPIonAbundanceRatioError(expand.getIonAbundanceRatioError().toString());
                            dto.setPIonAbundanceRatioWithinError(expand.getIonAbundanceRatioWithinError());
                            dto.setPIsDetected(dto.getIsDetected() == 1 ? "是" : "否");
                            dataDtos.add(dto);
                        }
                    }
                }
            }
        }

        // 9️⃣  返回处理后的数据
        data.put("dataDtos", dataDtos);
        return data;
    }


    public void buildIonPairAndCE(Map<String, Object> data, List<String> reagentIdList) {
        List<IonPairAndCEVO> ionPairAndCEVOS = new ArrayList<>();

        // 遍历 reagentIdList
        for (String id : reagentIdList) {
            TestRecordReagentVO vo = testRecordReagentService.getVOById(id);
            if (vo == null) {
                continue;  // 如果 vo 为空，跳过当前循环
            }

            Drug drug = vo.getDrug();
            if (drug == null) {
                continue;  // 如果 drug 为空，跳过当前循环
            }

            // 添加 ionPairAndCEVO1 和 ionPairAndCEVO2
            ionPairAndCEVOS.add(createIonPairAndCEVO(drug.getName(), drug.getMainProductIon(),
                    drug.getMainDeclusteringPotential(), drug.getMainCollisionEnergy()));
            ionPairAndCEVOS.add(createIonPairAndCEVO(drug.getName(), drug.getMinorProductIon(),
                    drug.getMinorDeclusteringPotential(), drug.getMinorCollisionEnergy()));
        }

        // 将结果放入数据 map 中
        data.put("ionPairAndCEVOS", ionPairAndCEVOS);
        data.put("ionPairAndCEVOSize", ionPairAndCEVOS.size() / 2);
    }

    // 创建 IonPairAndCEVO 对象的辅助方法
    private IonPairAndCEVO createIonPairAndCEVO(String compoundName, String productIon,
                                                Integer declusteringPotential, Integer collisionEnergy) {
        IonPairAndCEVO ionPairAndCEVO = new IonPairAndCEVO();
        ionPairAndCEVO.setCompoundName(compoundName);
        ionPairAndCEVO.setProductIon(productIon);
        ionPairAndCEVO.setDeclusteringPotential(declusteringPotential);
        ionPairAndCEVO.setCollisionEnergy(collisionEnergy);
        return ionPairAndCEVO;
    }


    /**
     * 根据委托信息和样本类型生成生物样本检验记录文件。
     *
     * @param entrustInfo  委托信息对象
     * @param materialType 样本类型（如"毛发"、"尿液"等）
     * @return 生成的生物样本检验记录文件的路径
     * @throws Exception 如果文件生成过程中出现错误，抛出异常
     */
    public String buildInVivoDocFile(EntrustInfo entrustInfo, String materialType) throws Exception {
        // 获取文件数据map
        Map<String, Object> data = this.buildInVivoRecordData(entrustInfo, materialType);
        // 调用模板生成检验记录
        String templatePath = "";
        if (materialType.equals("毛发")) {
            templatePath = "/template" + "/贵阳生物样本毛发检验记录模板.docx";
        } else {
            templatePath = "/template" + "/贵阳生物样本尿液检验记录模板.docx";
        }
        System.out.println(data);
        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();
        Configure config = Configure.builder().
                bind("dataDtos", policy).build();
        return buildDocFileAndUploadToOss(entrustInfo.getId(), templatePath, config, data);
    }

    /**
     * 根据委托信息、模板路径、配置和数据构建文档文件并上传到OSS
     *
     * @param entrustId    委托id
     * @param templatePath 模板文件路径
     * @param config       配置信息
     * @param data         要填充到模板中的数据
     * @return 上传后的文件路径
     * @throws Exception 可能抛出的异常
     */
    private String buildDocFileAndUploadToOss(String entrustId, String templatePath, Configure
            config, Map<String, Object> data) throws Exception {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ossFile.fileGet(templatePath, bos);
        byte[] templateArray = bos.toByteArray();
        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        XWPFTemplate template = XWPFTemplate.compile(bis, config).render(data);
        NiceXWPFDocument document = template.getXWPFDocument();

//        XWPFTable table = document.getTables().get(3);
//        int sampleSize = (int) data.get("sampleSize");
//        //合并单元格
//
//        TableTools.mergeCellsVertically(table, 1, 1, sampleSize);

        bis.close();
        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();
        document.close();
        String path = TestRecordFileUrl.TEST_RECORD_CATALOGUE.getFileUrl() + "/" + entrustId + "/" + "贵阳生物样本检验记录.docx";
        ossFile.fileSave(path, fisWord);
        return path;
    }


    /**
     * 生成常规毒品的检验记录（贵阳禁毒）
     */
    public String generateCommonDrugInpectRecord(String businessId) throws Exception {
        Map<String, Object> docMap = buildCommonDrugDocMap(businessId);
        EntrustInfo entrustInfo = entrustInfoService.getById(businessId);
        TestRecord testRecord = testRecordService.lambdaQuery()
                .eq(TestRecord::getBusinessId, businessId)
                .one();
        Map<String, Object> data = this.buildInVitroRecordData(entrustInfo, testRecord);
        docMap.putAll(data);
        String templatePath = "/template" + "/贵阳禁毒常规毒品检验记录模板.docx";
        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();
        Configure config = Configure.builder().
                bind("dataDtos", policy)
                .build();
        return buildDocFileAndUploadToOss(businessId, templatePath, config, docMap);
    }

    /**
     * 构建常规毒品的检验记录数据
     *
     * @return
     */
    private Map<String, Object> buildCommonDrugDocMap(String businessId) {
        EntrustInfo entrustInfo = entrustInfoService.getById(businessId);

        // 这里因为检验记录实体并未关联业务id, 只能先查询实验数据，在通过实验数据获取到检验记录信息
        TestRecord testRecord = testRecordService.getTestRecordByBusinessId(entrustInfo.getId());

        // 封装数据到map中
        Map<String, Object> docMap = buildCommonInspectRecordDocMap(entrustInfo, testRecord, "");


        return docMap;
    }


    /**
     * 构建通用的检验记录文档映射
     *
     * @param entrustInfo  委托信息对象
     * @param materialType
     * @return 包含委托信息和检验记录信息的HashMap对象
     */
    private HashMap<String, Object> buildCommonInspectRecordDocMap(EntrustInfo entrustInfo, TestRecord
            testRecord, String materialType) {
        HashMap<String, Object> data = new HashMap<>();

        data.put("entrustDepartment", entrustInfo.getEntrustDepartment());
        data.put("deliver1Name", entrustInfo.getDeliver1Name());
        data.put("deliver2Name", entrustInfo.getDeliver2Name());
        // 受理日期
        data.put("acceptYear", entrustInfo.getAcceptDate().getYear());
        data.put("acceptMonth", entrustInfo.getAcceptDate().getMonthValue());
        data.put("acceptDay", entrustInfo.getAcceptDate().getDayOfMonth());

        // 检验日期
        data.put("inspectYear", testRecord.getTestStartDate().getYear());
        data.put("inspectMonth", testRecord.getTestStartDate().getMonthValue());
        data.put("inspectDay", testRecord.getTestStartDate().getDayOfMonth());

        data.put("acceptNo", entrustInfo.getAcceptNo());

        // 检材性状描述和检验要求成分
        List<SampleInfo> sampleInfoList = getSampleInfosByMaterialType(entrustInfo, materialType);
        String materialCharacterDesc = buildMaterialCharacterDesc(sampleInfoList);
        String targetObjectStr = sampleInfoList
                .stream()
                .flatMap(
                        sampleInfo -> {
                            String toJSONString = JSONArray.toJSONString(sampleInfo.getTargetObject());
                            List<TargetObject> drugLiteList = JSONArray.parseArray(toJSONString, TargetObject.class);
                            return drugLiteList.stream();
                        })
                .collect(Collectors.toList()) // 得到转换好的筛查物对象
                .stream()
                .map(TargetObject::getName)
                .collect(Collectors.toSet()) // 区筛查物对象名称并去重
                .stream()
                .collect(Collectors.joining("、")); // 拼接筛查物名称
        data.put("materialCharacterDesc", materialCharacterDesc);
        data.put("materialIngredient", targetObjectStr);

        // 获取仪器设备数据（批量查询，减少数据库压力）
        List<String> deviceIdList = testRecord.getDeviceIdList();
        List<TestRecordInstrument> instruments = CollectionUtils.isEmpty(deviceIdList)
                ? Collections.emptyList()
                : testRecordInstrumentService.list(Wrappers.<TestRecordInstrument>lambdaQuery()
                .in(TestRecordInstrument::getId, deviceIdList));

        data.put("instrumentName", CollectionUtils.isEmpty(instruments)
                ? "未找到仪器设备数据!"
                : instruments.stream()
                .map(i -> "\u3000\u3000" + i.getInstrumentName()) // 使用 Unicode 全角空格
                .collect(Collectors.joining("\n")));


        // 设置使用的标准物质和试剂
        List<TestRecordReagent> references = testRecordReagentService.list(Wrappers.<TestRecordReagent>lambdaQuery()
                .in(TestRecordReagent::getId, testRecord.getReagentConsumablesList())
                .eq(TestRecordReagent::getCategory, "标准物质"));

        if (references == null || references.isEmpty()) {
            data.put("referenceMaterialName", "未找到试剂耗材数据!");
        } else {

            String referenceMaterialName = references.stream()
                    .map(reagent -> "\u2611" + " " + reagent.getReagentConsumableName())
                    .collect(Collectors.joining("\n"));

            data.put("referenceMaterialName", referenceMaterialName);
        }

        List<TestRecordReagent> reagents = testRecordReagentService.list(Wrappers.<TestRecordReagent>lambdaQuery()
                .in(TestRecordReagent::getId, testRecord.getReagentConsumablesList())
                .eq(TestRecordReagent::getCategory, "试剂"));

        if (reagents == null || reagents.isEmpty()) {
            data.put("reagentConsumableName", "未找到试剂耗材数据!");
        } else {
            String reagentConsumableName = reagents.stream()
                    .map(TestRecordReagent::getReagentConsumableName)
                    .collect(Collectors.joining("、"));
            data.put("reagentConsumableName", reagentConsumableName);
        }
        return data;
    }

    /**
     * 根据委托信息和检材类型获取检材信息列表
     *
     * @param entrustInfo  委托信息对象
     * @param materialType 检材类型, 毛发或尿液
     * @return 检材信息列表
     */
    private List<SampleInfo> getSampleInfosByMaterialType(EntrustInfo entrustInfo, String materialType) {
        List<SampleInfo> sampleInfoList = sampleInfoService.list(Wrappers.<SampleInfo>lambdaQuery().eq(SampleInfo::getBusinessId, entrustInfo.getId()));
        // 对检材进行筛选，如果materialType 不为空则进行操作
        if (StrUtil.isNotBlank(materialType)) {
            if (materialType.equals("毛发")) {
                sampleInfoList = sampleInfoList.stream()
                        .filter(o -> EntrustBiologyType.HAIR.equals(EntrustBiologyType.isExist(o.getForm())))
                        .collect(Collectors.toList());

            } else {
                // 尿液
                sampleInfoList = sampleInfoList.stream()
                        .filter(o -> EntrustBiologyType.URINE.equals(EntrustBiologyType.isExist(o.getForm())))
                        .collect(Collectors.toList());
            }
        }
        return sampleInfoList;
    }


    /**
     * 构建检材性状描述
     *
     * @param sampleInfoList 样本信息列表
     * @return 物料特性描述，如果无法构建则返回null
     */
    private String buildMaterialCharacterDesc(List<SampleInfo> sampleInfoList) {
        if (CollUtil.isEmpty(sampleInfoList)) {
            return StrUtil.EMPTY;
        }

        // 仅有一个样本时，直接返回
        if (sampleInfoList.size() == 1) {
            return "检材为" + sampleInfoList.get(0).getForm() + "。";
        }

        // 按 form 分组
        Map<String, List<SampleInfo>> sampleMapGroupByForm = sampleInfoList.stream()
                .collect(Collectors.groupingBy(SampleInfo::getForm));

        // 所有样本 form 相同
        if (sampleMapGroupByForm.size() == 1) {
            String form = sampleMapGroupByForm.keySet().iterator().next(); // 取唯一的 key
            String orderNoStr = sampleInfoList.stream()
                    .map(sample -> sample.getOrderNo() + "号")
                    .collect(Collectors.joining("、"));
            return orderNoStr + "检材均为" + form + "。";
        }

        // 存在多个 form 的情况
        List<String> descriptions = new ArrayList<>();
        sampleMapGroupByForm.forEach((form, value) -> {
            String orderNoStr = value.stream()
                    .map(sample -> sample.getOrderNo() + "号")
                    .collect(Collectors.joining("、"));
            descriptions.add(orderNoStr + "检材均为" + form);
        });

        return String.join("；", descriptions) + "。"; // 用分号连接不同类型的描述
    }

    public String buildInspectOpinion(List<TestRecordSampleData> dataList) {
        // 1. 检查输入的dataList是否为空，如果为空直接返回当前数据。
        if (dataList == null || dataList.isEmpty()) {
            return "";
        }

        // 2. 初始化数据结构，用于分类和记录检出与未检出的物质
        // - detectedMap 用来存储检出物质的样本信息，key为样本编号，value为检出的物质列表
        // - notDetectedMap 用来存储未检出物质的样本信息，key为样本编号，value为未检出的物质列表
        // - allCompoundNames 用来存储所有物质名称的集合，方便后续处理
        Map<String, List<String>> detectedMap = new LinkedHashMap<>();
        Map<String, List<String>> notDetectedMap = new LinkedHashMap<>();
        Set<String> allCompoundNames = new HashSet<>();

        // 3. 遍历dataList，按每个样本的检出状态（是否检出）对物质进行分类
        for (TestRecordSampleData record : dataList) {
            String sampleNo = record.getSampleNo(); // 获取样本编号
            String compoundName = record.getCompoundName(); // 获取化合物名称
            allCompoundNames.add(compoundName); // 将化合物名称添加到所有化合物名称集合中

            // 根据isDetected值将物质添加到相应的Map中
            if (record.getIsDetected() == 1) {
                detectedMap.computeIfAbsent(sampleNo, k -> new ArrayList<>()).add(compoundName); // 如果检出，将物质加入到检测的Map中
            } else {
                notDetectedMap.computeIfAbsent(sampleNo, k -> new ArrayList<>()).add(compoundName); // 如果未检出，将物质加入到未检测的Map中
            }
        }

        // 4. 定义一个lambda函数，提取样本编号中的流水号（例如："S001" -> "1号"）
        // 目的是生成以“号”结尾的样本编号格式。
        Function<String, String> extractSampleNumber = sampleNo -> sampleNo.substring(sampleNo.lastIndexOf('-') + 1) + "号";

        // 5. 处理“检出”数据
        // 先根据样本编号分组相同的检出物质，之后生成相应的语句描述
        List<String> detectedSentences = new ArrayList<>(); // 存储“检出”物质的描述
        Map<Set<String>, List<String>> groupedDetectedSamples = new LinkedHashMap<>(); // 用来存储按物质分组的样本编号

        // 遍历检测到的物质，将相同物质的样本编号进行分组
        for (Map.Entry<String, List<String>> entry : detectedMap.entrySet()) {
            Set<String> compoundSet = new HashSet<>(entry.getValue()); // 去重，保证每个物质只记录一次
            groupedDetectedSamples.computeIfAbsent(compoundSet, k -> new ArrayList<>()).add(extractSampleNumber.apply(entry.getKey())); // 根据物质的组合来分组样本编号
        }

        // 遍历每个分组，生成对应的描述
        for (Map.Entry<Set<String>, List<String>> entry : groupedDetectedSamples.entrySet()) {
            String sampleNumbers = String.join("、", entry.getValue()); // 将所有样本编号合并为字符串
            String compounds = String.join("、", entry.getKey()); // 将所有检出的物质合并为字符串

            // 如果多个样本检出相同的物质，描述中使用“均检出”，否则使用“检出”
            if (entry.getValue().size() > 1) {
                detectedSentences.add(String.format("从%s检材样品中均检出%s成分", sampleNumbers, compounds)); // 多个样本检出
            } else {
                detectedSentences.add(String.format("从%s检材样品中检出%s成分", sampleNumbers, compounds)); // 单个样本检出
            }
        }

        // 6. 处理“未检出”数据
        // 先根据未检出的物质及样本编号进行分组，然后生成相应的描述
        List<String> notDetectedSentences = new ArrayList<>(); // 存储“未检出”物质的描述
        Map<Set<String>, List<String>> groupedNotDetectedSamples = new LinkedHashMap<>(); // 用来存储按物质分组的样本编号

        // 遍历未检出的物质，找到未检出的样本并将其分组
        for (Map.Entry<String, List<String>> entry : notDetectedMap.entrySet()) {
            String sampleNo = entry.getKey(); // 获取样本编号
            if (detectedMap.containsKey(sampleNo)) {
                continue; // 如果该样本已经出现在检出组中，跳过，不再处理
            }

            Set<String> compoundSet = new HashSet<>(entry.getValue()); // 获取未检出的物质，去重
            groupedNotDetectedSamples.computeIfAbsent(compoundSet, k -> new ArrayList<>()).add(extractSampleNumber.apply(sampleNo)); // 将未检出的物质分组
        }

        // 遍历每个未检出物质的分组，生成描述
        for (Map.Entry<Set<String>, List<String>> entry : groupedNotDetectedSamples.entrySet()) {
            String sampleNumbers = String.join("、", entry.getValue()); // 将未检出的样本编号合并为字符串
            String compounds = String.join("、", entry.getKey()); // 将未检出的物质合并为字符串

            // 如果多个样本未检出相同的物质，描述中使用“均未检出”，否则使用“未检出”
            if (entry.getValue().size() > 1) {
                notDetectedSentences.add(String.format("从%s检材样品中均未检出%s成分", sampleNumbers, compounds)); // 多个样本未检出
            } else {
                notDetectedSentences.add(String.format("从%s检材样品中未检出%s成分", sampleNumbers, compounds)); // 单个样本未检出
            }
        }

        // 7. 将“检出”和“未检出”描述合并到最终结果中
        List<String> finalSentences = new ArrayList<>(detectedSentences); // 先将检出物质的描述加入
        finalSentences.addAll(notDetectedSentences); // 将未检出的描述添加到末尾

        // 8. 将最终的描述字符串添加到结果数据Map中，字段名为"detectionStr"返回结果
        return String.join("；", finalSentences);
    }
}
